# Endpoints for the API
CATEGORIES = "categories.json"
RECOMMENDATIONS = "get-recommendation-ads"
PRODUCT = "advert/{id}.json"
TRENDING = "recommended.json"
SEARCH = "ads_listing{page}"


USER_AGENT = "Android/JiJi/4.77.2/27"
